#include <msp/io/print.h>
#include <msp/strings/lexicalcast.h>
#include <msp/strings/utils.h>
#include "booleanevaluator.h"
#include "builder.h"
#include "package.h"

using namespace std;
using namespace Msp;

Package::Package(Builder &b, const string &n):
	builder(b),
	name(n),
	label(string(1, toupper(n[0]))+n.substr(1)),
	prepared(false),
	use_pkgconfig(true)
{
	builder.get_package_manager().add_package(this);
}

void Package::prepare()
{
	if(prepared)
		return;

	for(Requirements::const_iterator i=requires.begin(); i!=requires.end(); ++i)
		(*i)->prepare();

	do_prepare();
	prepared = true;
}


Package::Loader::Loader(Package &p):
	DataFile::ObjectLoader<Package>(p)
{
	add("if_arch", &Loader::if_arch);
	add("label",   &Package::label);
	add("require", &Loader::require);
}

void Package::Loader::if_arch(const string &cond)
{
	BooleanEvaluator eval(sigc::hide<1>(sigc::mem_fun(&obj.builder.get_current_arch(), &Architecture::match_name)), false);
	bool match = eval.evaluate(cond);
	obj.builder.get_logger().log("configure", format("%s: arch %s %smatched", obj.name, cond, (match ? "" : "not ")));
	if(match)
		load_sub_with(*this);
}

void Package::Loader::require(const string &n)
{
	Package *req = obj.builder.get_package_manager().find_package(n);
	if(req)
		obj.requires.push_back(req);
	else
		obj.problems.push_back(format("Required package %s not found", n));
}
