#include <msp/fs/utils.h>
#include <msp/strings/format.h>
#include "binarycomponent.h"
#include "builder.h"
#include "objectfile.h"
#include "sharedlibrary.h"
#include "sourcepackage.h"

using namespace std;
using namespace Msp;

SharedLibrary::SharedLibrary(Builder &b, const Msp::FS::Path &p):
	Binary(b, p),
	import_lib(0)
{
	libname = FS::basepart(FS::basename(path));
	if(!libname.compare(0, 3, "lib"))
		libname = libname.substr(3);
}

SharedLibrary::SharedLibrary(Builder &b, const Component &c, const list<ObjectFile *> &objs):
	Binary(b, c, generate_filename(c), objs),
	libname(c.get_name()),
	import_lib(0)
{
	if(builder.get_current_arch().get_system()=="windows")
		install_location = "bin";
	else
		install_location = "lib";

	const BinaryComponent &bcomp = dynamic_cast<const BinaryComponent &>(*component);
	if(bcomp.get_type()==BinaryComponent::MODULE)
		install_location /= package->get_name();
	else
	{
		const string &version = component->get_package().get_interface_version();
		if(!version.empty())
		{
			const Architecture &arch = builder.get_current_arch();
			const Pattern &pattern = arch.get_shared_library_patterns().front();
			if(arch.get_system()=="windows")
				soname = pattern.apply(format("%s-%s", component->get_name(), version));
			else if(arch.get_system()=="darwin")
			{
				string filename = pattern.apply(component->get_name());
				string base = FS::basepart(filename);
				string ext = FS::extpart(filename);
				soname = format("%s.%s%s", base, version, ext);
			}
			else
				soname = format("%s.%s", pattern.apply(component->get_name()), version);

			install_filename = soname;
		}
	}

	for(list<ObjectFile *>::const_iterator i=objects.begin(); i!=objects.end(); ++i)
		(*i)->set_used_in_shared_library(true);
}

string SharedLibrary::generate_filename(const Component &comp)
{
	const BinaryComponent &bcomp = dynamic_cast<const BinaryComponent &>(comp);
	if(bcomp.get_type()==BinaryComponent::MODULE)
		return comp.get_name()+".m";
	else
	{
		const Architecture &arch = comp.get_package().get_builder().get_current_arch();
		return arch.get_shared_library_patterns().front().apply(comp.get_name());
	}
}

void SharedLibrary::set_import_library(ImportLibrary *imp)
{
	import_lib = imp;
}
